import React, { useState, useEffect } from 'react';
import toast from 'react-hot-toast';
import { useSelector } from 'react-redux';
import styles from './PricingForm.module.css';
import config from '../../../../../config';
import { getAuthToken } from '../../../../utils/auth';

const getCategories = async (publisherId) => {
  const accessToken = getAuthToken();

  const response = await fetch(
    `${config.ApiBaseUrl}/publisher/${publisherId}/articleCategories`,
    {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${accessToken}`,
      },
    }
  );
  const json = await response.json();

  const { data, message, success } = json;

  if (!success)
    throw new Error(
      message ?? 'Something went wrong. failed to process request'
    );

  const retrievedCategories = data.articleCategories.map((item) => ({
    category: item.category,
    id: item.id,
    price: item.price,
  }));

  return retrievedCategories;
};

const putCategoryPrice = async (publisherId, categoryId, price) => {
  const accessToken = getAuthToken();

  const response = await fetch(
    `${config.ApiBaseUrl}/publisher/${publisherId}/articleCategory/${categoryId}/price/${price}`,
    {
      method: 'PUT',
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${accessToken}`,
      },
    }
  );
  const json = await response.json();

  const { message, success } = json;

  if (!success)
    throw new Error(
      message ?? 'Something went wrong. failed to process request'
    );

  return success;
};

const PricingForm = () => {
  const [categories, setCategories] = useState([]);
  const [selectedCategory, setSelectedCategory] = useState({
    name: 'none',
    id: -1,
  });
  const [price, setPrice] = useState('');
  // const accessToken = getAuthToken();
  const publisherId = useSelector((state) => state.user.publisher?.id);
  // const userData = useSelector((state) => state.user.user);

  useEffect(() => {
    if (publisherId) {
      getCategories(publisherId).then((retrievedCategories) => {
        setCategories(retrievedCategories);
      });
    }
  }, [publisherId]);

  function handleCategoryChange(event) {
    const name = event.target.value;
    const id = event.target.options[event.target.selectedIndex]?.id;

    console.log(name, id);

    setSelectedCategory({
      name,
      id,
    });
  }

  function priceChangeHandler(event) {
    setPrice(event.target.value);
  }

  function submitPrice(event) {
    event.preventDefault();

    toast.promise(putCategoryPrice(publisherId, selectedCategory.id, price), {
      loading: 'setting category price...',
      success: 'category price set!',
      error: 'Error while setting category price!',
    });
  }

  const TransformedCategories = [
    ...new Set(categories.map(({ category, id }) => ({ category, id }))),
  ];

  return (
    <>
      <div className={styles.category_selector}>
        <form>
          <label htmlFor="category" id={styles.category_input}>
            Choose your category
          </label>
          <div id={styles.catergory_input_items}>
            <select id={styles.cat} name="cat" onChange={handleCategoryChange}>
              <option value="none" selected disabled hidden>
                Select an Option
              </option>
              {TransformedCategories.map(({ category, id }) => (
                <option value={category} id={id}>
                  {category}
                </option>
              ))}
            </select>
            <input
              type="number"
              min="0.5"
              step="0.01"
              id={styles.usd}
              onChange={priceChangeHandler}
            />
            <label htmlFor="usd" id={styles.usdLabel}>
              USD
            </label>
          </div>
        </form>
        <button
          type="submit"
          onClick={submitPrice}
          id={styles.form__submit}
          disabled={price === '' || selectedCategory.id === -1}
        >
          save price
        </button>
      </div>
    </>
  );
};

export default PricingForm;
